# -*- coding: utf-8 -*-
# vim: ts=4 
###
#
# CommieCC is the legal property of J. Félix Ontañón <felixonta@gmail.com>
# Copyright (c) 2009 J. Félix Ontañón
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###

import dbus 
import threading

PROP_IFACE_URI = 'org.freedesktop.DBus.Properties'

NM_URI = 'org.freedesktop.NetworkManager'
NM_PATH = '/org/freedesktop/NetworkManager'
NM_IFACE_URI = NM_URI
NM_ACTIVE_CONN_IFACE_URI = NM_URI + '.Connection.Active'
NM_DEVICE_IFACE_URI = NM_URI + '.Device'
NM_WIRED_IFACE_URI = NM_DEVICE_IFACE_URI + '.Wired'
NM_WIRELESS_IFACE_URI = NM_DEVICE_IFACE_URI + '.Wireless'

NM_SETTINGS_URI = 'org.freedesktop.NetworkManagerSettings'
NM_SETTINGS_PATH = '/org/freedesktop/NetworkManagerSettings'
NM_SETTINGS_IFACE_CONN_URI = NM_SETTINGS_URI + '.Connection'

def get_active_ifaces():
    iface = {}
    bus = dbus.SystemBus()
    nm_iface = dbus.Interface(bus.get_object(NM_URI, NM_PATH), 
        dbus_interface=PROP_IFACE_URI)

    active_conns_path = nm_iface.Get(NM_IFACE_URI, 'ActiveConnections')
            
    for active_conn_path in active_conns_path:
        active_conn_iface = dbus.Interface(bus.get_object(NM_URI, 
            active_conn_path), dbus_interface=PROP_IFACE_URI)
            
        active_devs_path = active_conn_iface.Get(NM_ACTIVE_CONN_IFACE_URI, 'Devices')
        
        for active_dev_path in active_devs_path:
            active_dev_iface = dbus.Interface(bus.get_object(NM_URI,
                active_dev_path), dbus_interface=PROP_IFACE_URI)
            
            devicetype = active_dev_iface.Get(NM_DEVICE_IFACE_URI, 'DeviceType')
            interface = active_dev_iface.Get(NM_DEVICE_IFACE_URI, 'Interface')
            
            #Wired
            if devicetype == 1:
                hwaddr = active_dev_iface.Get(NM_WIRED_IFACE_URI, 'HwAddress')
                
            #Wireless
            elif devicetype == 2:
                hwaddr = active_dev_iface.Get(NM_WIRELESS_IFACE_URI, 'HwAddress')
                
            iface[str(interface)] = str(hwaddr)
           
    return iface

def threaded(f):
    def wrapper(*args):
        t = threading.Thread(target=f, args=args)
        t.setDaemon(True)
        t.start()
    return wrapper

# Sets up the main logger
import logging

LOG_MAIN = 'main_log'
LOG_FILE = '/var/log/commiecc-slave.log'
LOG_LEVEL = logging.DEBUG
LOG_FORMAT = '%(asctime)s %(levelname)s - %(message)s'
LOG_DATE_FORMAT = '%H:%M:%S'

def setup_main_logger():
    import logging.handlers

    main_logger = logging.getLogger(LOG_MAIN)
    main_logger.setLevel(LOG_LEVEL)
    handler = logging.handlers.RotatingFileHandler(LOG_FILE, maxBytes=300000, 
        backupCount=5)
    handler.setFormatter(logging.Formatter(LOG_FORMAT, datefmt=LOG_DATE_FORMAT))
    main_logger.addHandler(handler)

class log():
    def __init__(self, msg, level=logging.DEBUG):
        self.msg = msg
        self.level = level
        self.logger = logging.getLogger(LOG_MAIN)
        
    def __call__(self, f):
        def wrapped_f(*args):
            self.logger.log(self.level, self.msg)        
            f(*args)
        return wrapped_f
